/*
 * Segment 2 (KEx),  SWE 2.1 LMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lms;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

import com.bitctrl.resource.Configuration;
import com.bitctrl.util.logging.LoggerTools;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.MutableSetChangeListener;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.bitctrl.modell.ObjektFactory;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * L&ouml;scht alle vorhandenen RDS-Meldungen.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */
public class MeldungenLoeschen implements MutableSetChangeListener {

	/** Der Logger der Klassse. */
	private static Logger log = Logger.getLogger(MeldungenLoeschen.class.getName());

	private static ClientDavConnection verbindung;

	/**
	 * Initialisiert den Testfall.
	 *
	 * @throws Exception
	 *             bei einem Fehler.
	 */
	@BeforeClass
	public static void beforeClass() throws Exception {
		final ObjektFactory factory;

		final Configuration konfig;
		final Level level;

		konfig = Configuration.getConfiguration();
		level = Level.parse(konfig.getString("debug.level"));
		LoggerTools.setLoggerLevel(level);
		MeldungenLoeschen.log.setLevel(level);

		final String[] args = { "-debugLevelStdErrText=INFO", "-debugLevelFileText=INFO" };
		final ArgumentList al = new ArgumentList(args);

		MeldungenLoeschen.log.info("Stelle Verbindung zum Datenverteiler her ...");
		Debug.init("MeldungeLoeschen", al);
		MeldungenLoeschen.verbindung = new ClientDavConnection();
		MeldungenLoeschen.verbindung.getClientDavParameters().setDavCommunicationAddress(konfig.getString("dav.host"));
		MeldungenLoeschen.verbindung.getClientDavParameters().setDavCommunicationSubAddress(konfig.getInt("dav.port"));
		MeldungenLoeschen.verbindung.connect();
		MeldungenLoeschen.verbindung.login(konfig.getString("dav.benutzer"), konfig.getString("dav.kennwort"));

		factory = ObjektFactory.getInstanz();
		factory.setVerbindung(MeldungenLoeschen.verbindung);
		factory.registerStandardFactories();
	}

	/**
	 * Setup-Methode.
	 *
	 * @throws Exception
	 *             bei Ausnahmen
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		System.out.println("Alle RDS-Meldungen werden gelöscht!");
	}

	/**
	 * Liste der Meldungen.
	 */
	List<SystemObject> meldungen;

	/**
	 * Lock-Objekt.
	 */
	private Object myLock;

	/**
	 * L&ouml;scht alle RDS-Meldungen.
	 */
	@Test
	public void loescheMeldungen() {

		final SystemObject lms = MeldungenLoeschen.verbindung.getDataModel().getObject("lmst.bw");
		Assert.assertNotNull(lms);
		try {
			final MutableSet meldungsmenge = ((ConfigurationObject) lms).getMutableSet("RDSMeldungen");
			Assert.assertNotNull(meldungsmenge);
			meldungsmenge.addChangeListener(this);
			System.out.println("Alle RDS-Meldungen am Objekt '" + lms + "' werden gelöscht!");
			meldungen = meldungsmenge.getElements();
			if (meldungen.size() != 0) {
				final List<SystemObject> mi = new ArrayList<SystemObject>(meldungen);
				for (final SystemObject object : mi) {
					System.out.println("Meldung " + object + " wird gelöscht!");
					meldungsmenge.remove(object);
				}

				myLock = new Object();

				synchronized (myLock) {
					try {
						myLock.wait(10000);
					} catch (final InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
				Assert.assertTrue(meldungen.size() == 0);
				Assert.assertTrue(meldungsmenge.getElements().size() == 0);
			} else {
				System.out.println("keine Meldungen aus der Menge zu löschen");
			}

			// RDS-Meldungen, die nicht in der Menge enthalten sind:
			final List<SystemObject> robjekte = MeldungenLoeschen.verbindung.getDataModel().getType("typ.rdsMeldung")
					.getElements();
			System.out.println("lösche weitere " + robjekte.size() + " RDS-Meldungen");
			for (final SystemObject so : robjekte) {
				System.out.println("Meldung " + so + " wird gelöscht!");
				so.invalidate();
			}

		} catch (final Exception e) {
			e.printStackTrace();
		}
	}

	@Override
	public void update(final MutableSet set, final SystemObject[] addedObjects, final SystemObject[] removedObjects) {
		System.out.println("gelöschte Objekte: " + removedObjects);
		for (final SystemObject so : removedObjects) {
			meldungen.remove(so);
			try {
				so.invalidate();
			} catch (final ConfigurationChangeException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		if ((meldungen.size() == 0) && (myLock != null)) {
			synchronized (myLock) {
				myLock.notify();
			}
		}
	}

}
