/*
 * Segment 2 (KEx),  SWE 2.1 LMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lms.empfang.dekoder;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.bitctrl.Constants;
import com.bitctrl.util.logging.LoggerTools;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.InvalidationListener;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.MutableSetChangeListener;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.kex.lms.empfang.ftpserver.LMSFTPServer;
import de.bsvrz.kex.lms.vew.LMSFTPListener;
import de.bsvrz.kex.lms.vew.LMSKommunikationsstatus;
import de.bsvrz.kex.lms.vew.LMSParameter;
import de.bsvrz.kex.lms.vew.LMSProperties;
import de.bsvrz.kex.lms.vew.LmsMsg;
import de.bsvrz.kex.lms.vew.RDSException;
import de.bsvrz.kex.lms.vew.RDSMeldung;
import de.bsvrz.kex.lms.vew.daten.RDSMeldungsDaten;
import de.bsvrz.sys.funclib.bitctrl.daf.LogTools;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.dynobj.DynObjektException;
import de.bsvrz.sys.funclib.dynobj.DynamischeObjekte;

/**
 * Modul Meldungsdekodierer.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */

public class Meldungsdekodierer extends Thread
		implements ClientReceiverInterface, ClientSenderInterface, LMSFTPListener, InvalidationListener {

	/**
	 * Erweiterung einer ArrayList, die 'null' als Element nicht annimmt.
	 *
	 * @param <E>
	 */
	class ListeOhneNullElemente<E> extends ArrayList<E> {

		/**
		 * VersionUID.
		 */
		private static final long serialVersionUID = 1L;

		@Override
		public boolean add(final E o) {
			if (o != null) {
				return super.add(o);
			}

			return false;
		}
	}

	/**
	 * Listener f&uuml;r &Auml;nderungen der Menge der RDS-Meldungen des
	 * Landesmeldestelle.
	 */
	class MengeListener implements MutableSetChangeListener {

		@Override
		public void update(final MutableSet set, final SystemObject[] addedObjects,
				final SystemObject[] removedObjects) {
			LogTools.log(debug, LmsMsg.FineMengenAenderung);

			for (final SystemObject so : addedObjects) {
				LogTools.log(debug, LmsMsg.InfoMeldungNeu, so.getPid());
				hinzufuegenMeldung(so);
			}

			for (final SystemObject so : removedObjects) {
				LogTools.log(debug, LmsMsg.InfoMeldungEntfernt, so.getPid());
				entfernenMeldung(so);
			}
		}
	}

	/**
	 * Attributgruppe der Kommunikationsparameter.
	 */
	private static final String ATTRIBUTGRUPPE_PARAMETER = "atg.rdsKommunikationsParameter";

	/**
	 * DebugLogger f&uuml;r Debug-Ausgaben.
	 */
	final Debug debug = Debug.getLogger();

	/**
	 * DaV Interface.
	 */
	private ClientDavInterface dav = null;

	/**
	 * Datenmodell der Konfiguration.
	 */
	private DataModel config = null;

	// private final String DAV_TYP = "typ.landesMeldeStelle";

	/**
	 * Objektreferenz auf das Objekt vom Typ "typ.landesMeldeStelle".
	 */
	ConfigurationObject lms = null;

	/**
	 * Die Liste der zu empfangenen Meldungen.
	 */
	Map<String, RDSMeldung> meldungen = new HashMap<String, RDSMeldung>();

	/**
	 * Die Menge der RDSMeldungen der Landesmeldestelle.
	 */
	MutableSet meldungsmenge = null;

	/**
	 * Die aktuellen Parameter.
	 */
	private final LMSParameter lmsParameter = new LMSParameter();

	/** Datenbeschreibung, mit der Parameter empfangen werden. */
	private DataDescription paramEmpfang;

	/** Kommunikationsstatus. */
	private LMSKommunikationsstatus kommunikationsStatus;

	/** die Liste neu empfangener Dateien. */
	private final ArrayList<File> neueDateien = new ArrayList<File>();

	/** Map zur Zuordnung von Dateien zu enthaltenen Meldungen. */
	private final Map<File, List<RDSMeldung>> dateiMap = new HashMap<File, List<RDSMeldung>>();

	/** FTP-Serverthread. */
	private LMSFTPServer ftpserver;

	/**
	 * Run-Flag.
	 */
	boolean run = true;

	/** Klasse zum Verwalten dynamischer Objekte. */
	DynamischeObjekte vdo = null;

	/** Objekttyp der Meldungen. */
	DynamicObjectType objektTyp = null;

	/** Waitlock. */
	private final Object waitLock = new Object();

	/**
	 * Konstruktor.
	 *
	 * @param dav
	 *            Datenverteiler-Verbindung
	 * @param lms
	 *            Landesmeldestelle
	 */
	public Meldungsdekodierer(final ClientDavInterface dav, final ConfigurationObject lms) {
		super();
		this.dav = dav;
		this.lms = lms;
		config = this.dav.getDataModel();
		this.setName(this.getClass().getSimpleName());
		vdo = DynamischeObjekte.getInstanz(dav);
		objektTyp = (DynamicObjectType) dav.getDataModel().getType("typ.rdsMeldung");
		if (objektTyp == null) {
			throw new RuntimeException("Der Typ '" + "typ.rdsMeldung" + "' existiert nicht!");
		}

		// selbst starten
		start();
	}

	/**
	 * Aktualisiert eine bestehende Meldung.
	 *
	 * @param meldungsdaten
	 *            Meldungsdaten
	 * @return aktualisierte Meldung
	 * @throws RDSException
	 *             bei Ausnahmen
	 */
	private RDSMeldung aktualisiereMeldung(final RDSMeldungsDaten meldungsdaten) throws RDSException {
		final RDSMeldung rdsmeldung = meldungen.get(meldungsdaten.bildePid());

		if (rdsmeldung == null) {
			return null;
		}

		// übernehmen, wenn neuere oder gleiche Version (gleiche Version wird
		// beim Start
		// mit vohandenen Dateien benoetigt
		if (meldungsdaten.getVersion() >= rdsmeldung.getVersion()) {
			LogTools.log(debug, LmsMsg.InfoMeldungAktualisiert, rdsmeldung.getPID(), meldungsdaten.getVersion());
			rdsmeldung.setMeldungsDaten(meldungsdaten);
			rdsmeldung.publiziereDAV(dav);
			return rdsmeldung;
		}

		LogTools.log(debug, LmsMsg.InfoMeldungVerworfen, rdsmeldung.getPID());
		return null;
	}

	/**
	 * Kopiert den Dateiinhalt.
	 *
	 * @param in
	 *            Eingabestrom
	 * @param out
	 *            Ausgabestrom
	 * @throws IOException
	 *             bei Ausnahmen
	 */
	void copy(final InputStream in, final OutputStream out) throws IOException {
		final byte[] buffer = new byte[0xFFFF];

		for (int len; (len = in.read(buffer)) != -1;) {
			out.write(buffer, 0, len);
		}
	}

	@Override
	public void dataRequest(final SystemObject object, final DataDescription dataDescription, final byte state) {
		/**
		 * nichts zu tun
		 */
	}

	@Override
	public void empfangen(final List<File> dateien) {
		synchronized (neueDateien) {
			neueDateien.addAll(dateien);
			Collections.sort(neueDateien);
		}

		synchronized (waitLock) {
			waitLock.notify();
		}
	}

	/**
	 * Entfernt eine Meldung aus der Meldungsliste.
	 *
	 * @param so
	 *            Systemobjekt der Meldung
	 */
	synchronized void entfernenMeldung(final SystemObject so) {
		synchronized (meldungen) {
			final RDSMeldung rdsMeldung = meldungen.get(so.getPid());
			if (rdsMeldung != null) {
				try {
					rdsMeldung.abmeldenDavEmpfang(dav);
				} catch (final RDSException e) {
					debug.error("abmeldenDavEmpfang: " + e.getMessage());
				}
				meldungen.remove(so.getPid());
			}
		}
	}

	/**
	 * F&uuml;gt eine Meldung zur Meldungsliste hinzu.
	 *
	 * @param objekt
	 *            Systemobjekt der Meldung
	 *
	 */
	public void hinzufuegenMeldung(final SystemObject objekt) {
		synchronized (meldungen) {
			final RDSMeldung meldung = new RDSMeldung(objekt);
			try {
				meldung.aktualisierenDav(dav);
			} catch (final RDSException e) {
				// das ist hier kein Fehler:
				// Meldungen von RdcTmcApp, die erst in die Menge eingetragen
				// und dann mit Daten gefüttert werden erzeugen sonst hier
				// einen Fehler
				// LogTools.log(debug, LmsMsg.ErrorAppException,
				// e.getMessage());
			}
			try {
				meldung.anmeldenDavEmpfang(dav);
			} catch (final RDSException e) {
				LogTools.log(debug, LmsMsg.ErrorAppException, e.getMessage());
			}
			meldungen.put(meldung.getPID(), meldung);
		}
	}

	@Override
	public boolean isRequestSupported(final SystemObject object, final DataDescription dataDescription) {
		// debug.finest("isRequestSupported");
		return false;
	}

	/**
	 * Filterung eingehender Meldungen.<br>
	 * Alle eingehenden Meldungen werden auf Relevanz f&uuml;r die VRZ, f&uuml;r
	 * die diese SWE l&auml;uft, gepr&uuml;ft. Diese Pr&uuml;fung geschieht auf
	 * der Basis der Parameter
	 * <code>atg.rdsKommunikationsParameter.AkzeptierteLocationCodes</code> und
	 * <code>atg.rdsKommunikationsParameter.IgnorierteLocationCodes</code>. Eine
	 * empfangene RDS-Meldung wird unter folgender Bedingung als relevant
	 * betrachtet: <br>
	 * Die Meldung enth&auml;lt mindestens einen akzeptierten, nicht ignorierten
	 * LocationCode. <br>
	 * Eine leere Liste
	 * <code>atg.rdsKommunikationsParameter.AkzeptierteLocationCodes</code> wird
	 * als pauschale Akzeptanz interpretiert, d. h. alle LocationCodes werden
	 * akzeptiert.
	 *
	 * @param meldung
	 *            die zu pr&uuml;fende Meldung
	 * @return true, wenn die Meldung relevant ist, sonst false
	 */
	private boolean istRelevant(final RDSMeldungsDaten meldung) {
		if (lmsParameter.getAkzeptierteLocationCodes().size() == 0) {
			return true;
		}

		final List<Integer> meldungslocationen = meldung.getLocationCodes();

		for (final Integer loc : meldungslocationen) {
			if (lmsParameter.getAkzeptierteLocationCodes().contains(loc)
					&& !lmsParameter.getIgnorierteLocationCodes().contains(loc)) {
				return true;
			}
		}

		debug.info("Die Meldung " + meldung.getIdentifikation()
				+ " wird aufgrund enthaltener Locationcode(s) als nicht relevant erkannt");
		return false;
	}

	@Override
	public void kommunikationsStatus(final LMSKommunikationsstatus status) {
		kommunikationsStatus = status;
		if (kommunikationsStatus == LMSKommunikationsstatus.GESTOERT) {
			LogTools.log(debug, LmsMsg.WarningStoerungEingehend, lms.getName());
		}
		if (kommunikationsStatus == LMSKommunikationsstatus.NICHT_GESTOERT) {
			LogTools.log(debug, LmsMsg.InfoStatusEingehendOK, lms.getName());
		}

	}

	/**
	 * Kopiert eine Datei.
	 *
	 * @param quelldatei
	 *            Quelldatei
	 * @param zieldatei
	 *            Zieldatei
	 * @throws IOException
	 *             bei Ausnahmen
	 */
	void kopiereDatei(final String quelldatei, final String zieldatei) throws IOException {
		FileInputStream fis = null;
		FileOutputStream fos = null;

		try {
			fis = new FileInputStream(quelldatei);
			fos = new FileOutputStream(zieldatei);

			copy(fis, fos);
		} finally {
			if (fis != null) {
				fis.close();
			}
			if (fos != null) {
				fos.close();
			}
		}
	}

	/**
	 * Erzeugt ein dynamisches Objekt f&uuml;r die RDSMeldung.
	 *
	 * @param meldungsdaten
	 *            Meldungsdaten
	 * @return Systemobjekt
	 * @throws RDSException
	 *             bei Ausnahmen
	 */
	private SystemObject erzeugeMeldungsObjekt(final RDSMeldungsDaten meldungsdaten) throws RDSException {
		// ConfigurationArea area = dav.getLocalConfigurationAuthority()
		// .getConfigurationArea();

		final String pid = meldungsdaten.bildePid();
		SystemObject so = config.getObject(pid);

		if (so == null) {
			try {
				// Gieseler, 25.11.08: benutze de.bsvrz.sys.funclib.vewdynobj
				// zum Anlegen dynamischer
				// Objekte
				// so = area.createDynamicObject((DynamicObjectType) config
				// .getType("typ.rdsMeldung"), pid, pid);
				// if (vdo.erzeugeObjekt("typ.rdsMeldung", pid, pid, null)) {

				// so = config.getObject(pid);
				// meldungsmenge.add(so);
				// } else {
				// throw new RDSException(MessageFormat.format(
				// LmsMsg.ErrorErzeugeObjekt.toString(), pid));
				// }
				so = vdo.erzeugeObjektInMenge(objektTyp, pid, pid, meldungsmenge);
			} catch (final DynObjektException e) {
				throw new RDSException(e.getMessage());
			}
		} else {
			LogTools.log(debug, LmsMsg.WarningMeldungExistiert, pid);
		}

		return so;
	}

	/**
	 * Ezeugt ein dynamisches Objekt f&uuml;r die RDSMeldung.
	 *
	 * @param obj
	 *            Meldungsobjekt
	 *
	 * @throws RDSException
	 *             bei Ausnahmen
	 */
	private void loescheMeldungsObjekt(final SystemObject obj) throws RDSException {
		boolean ausMengeEntfernt = false;
		try {
			// meldungsmenge.remove(obj);
			// vdo.loescheObjekt(obj.getPid());
			ausMengeEntfernt = vdo.entferneObjektAusMenge((DynamicObject) obj, meldungsmenge, true);
		} catch (final DynObjektException e) {
			if (!ausMengeEntfernt) {
				LogTools.log(debug, LmsMsg.WarningMeldungNichtInMenge, obj.getPid());
				// trotzdem löschen
				try {
					vdo.loescheObjekt((DynamicObject) obj);
				} catch (final DynObjektException e1) {
					throw new RDSException(e.getMessage());
				}
			} else {
				throw new RDSException(e.getMessage());
			}
		}
	}

	/**
	 * Ezeugt ein neues Objekt vom Typ RDSMeldung.
	 *
	 * @param meldungsdaten
	 *            Meldungsdaten
	 * @return neue RDS-Meldung
	 * @throws RDSException
	 *             bei Ausnahmen
	 */
	private RDSMeldung neueMeldung(final RDSMeldungsDaten meldungsdaten) throws RDSException {
		final String pid = meldungsdaten.bildePid();

		LogTools.log(debug, LmsMsg.InfoMeldungErzeugt, pid, meldungsdaten.getVersion());

		LogTools.log(debug, LmsMsg.FineMeldungObjekt, pid);

		final SystemObject so = erzeugeMeldungsObjekt(meldungsdaten);

		RDSMeldung rdsmeldung = null;

		try {
			rdsmeldung = new RDSMeldung(so, meldungsdaten);
			LogTools.log(debug, LmsMsg.FineMeldungDatenAnmelden, pid);
			rdsmeldung.anmeldenDavEmpfang(dav);
			meldungen.put(rdsmeldung.getPID(), rdsmeldung);
			LogTools.log(debug, LmsMsg.FineMeldungDatenPublizieren, pid);
			rdsmeldung.publiziereDAV(dav);
		} catch (final RDSException rdse) {
			LogTools.log(debug, LmsMsg.ErrorAppException, rdse.getMessage());
			// Object wieder entfernen
			loescheMeldungsObjekt(so);
			throw rdse;
		}

		LogTools.log(debug, LmsMsg.InfoMeldungErzeugtFertig, pid);

		return rdsmeldung;
	}

	@Override
	public void run() {
		Thread.setDefaultUncaughtExceptionHandler(new Thread.UncaughtExceptionHandler() {
			@Override
			public void uncaughtException(final Thread t, final Throwable e) {
				LogTools.log(debug, LmsMsg.ErrorThreadBeendet, "Meldungsdekodierer (" + e + ")"); //$NON-NLS-1$
				run = false;
				if ((ftpserver != null) && ftpserver.isAlive()) {
					ftpserver.stopp();
					try {
						ftpserver.join();
					} catch (final InterruptedException e1) {
						// e1.printStackTrace();
					}
				}
			}
		});

		try {
			subscribeData();

			ftpserver = new LMSFTPServer(lmsParameter);
			ftpserver.addListener(this);
			while (run) {
				try {
					ueberpruefeDateien();
					ueberpruefeSicherungsverzeichnis();
					verarbeiteNeueDateien();

					synchronized (waitLock) {
						waitLock.wait(60000);
					}

					if (!ftpserver.isAlive()) {
						LogTools.log(debug, LmsMsg.ErrorThreadBeendet, "FTPServer");
						break;
					}
				} catch (final InterruptedException e) {
					// _debug.error("unterbrochen");
					// interrupt();
				}
			}
		} catch (final RDSException e) {
			LogTools.log(debug, LmsMsg.ErrorAppException, e.getMessage());
		}

		if (ftpserver != null) {
			ftpserver.stopp();
		}
	}

	/**
	 * Kopiert eine Datei in das Sicherungsverzeichnis.
	 *
	 * @param datei
	 *            die zusichernde Datei
	 */
	private void sichereDatei(final File datei) {
		final File sicherungsverzeichnis = new File(
				LMSProperties.getInstance().getProperty(LMSProperties.PROP_KEY_SICHERUNGSVERZEICHNIS));

		if (sicherungsverzeichnis.length() == 0) {
			// keine Sicherung
			return;
		}

		try {
			if (!sicherungsverzeichnis.exists()) {
				sicherungsverzeichnis.mkdirs();
			}
			final File backupfile = new File(sicherungsverzeichnis, datei.getName());

			debug.info("Die Datei " + datei + " wird nach " + backupfile.getAbsolutePath() + " kopiert");
			// es wird kopiert und nicht verschoben, da die Dateien in
			// verschiedenen Dateisystemen liegen können
			kopiereDatei(datei.getAbsolutePath(), backupfile.getAbsolutePath());
		} catch (final Exception e) {
			LogTools.log(debug, LmsMsg.ErrorAppException, e.getMessage());
			debug.error("Fehler beim Kopieren der Datei " + datei + e.getMessage());
		}
	}

	/**
	 * Stoppt den Thread.
	 */
	public void stopp() {
		this.run = false;
		this.interrupt();
	}

	/**
	 * Anmeldung aller Daten am DAV.
	 *
	 * @throws RDSException
	 *             bei Ausnahmen
	 */
	protected void subscribeData() throws RDSException {
		final DataDescription dataDescription = null;
		SystemObject[] objekte = null;
		AttributeGroup attributeGroup = null;
		Aspect aspect = null;

		// Anmeldungen zum Senden und Empfangen.
		try {
			// Menge RDS-Meldungen auslesen
			meldungsmenge = lms.getMutableSet("RDSMeldungen");
			if (meldungsmenge == null) {
				throw new RDSException(MessageFormat.format(LmsMsg.ErrorLMSObjektMenge.toString(), lms));
			}

			for (final SystemObject meldung : meldungsmenge.getElements()) {
				LogTools.log(debug, LmsMsg.InfoMeldungInitialisiert, meldung);
				hinzufuegenMeldung(meldung);
			}

			//
			// Anmeldungen zum Empfang
			//

			// Kommunikationsparameter
			objekte = new SystemObject[] { lms };
			attributeGroup = config.getAttributeGroup(Meldungsdekodierer.ATTRIBUTGRUPPE_PARAMETER);
			aspect = config.getAspect("asp.parameterSoll");
			paramEmpfang = new DataDescription(attributeGroup, aspect);
			debug.finer("Anmeldung zum Empfang von Daten beim DaV als SENKE :" + "(Objekt: " + lms.getName()
					+ ") (ATG: " + paramEmpfang.getAttributeGroup().getName() + ") (ASP: "
					+ paramEmpfang.getAspect().getName() + ") (OT: " + lms.getType().getName() + ")");

			dav.subscribeReceiver(this, objekte, paramEmpfang, new ReceiveOptions(false), // keine
					// Deltas,
					// keine
					// nachgelieferten Daten
					ReceiverRole.receiver());

			// IvalidationListener
			final DynamicObjectType typ = (DynamicObjectType) config.getType("typ.rdsMeldung");
			// typ.addObjectCreationListener(privateListener);
			typ.addInvalidationListener(this);

		} catch (final DataNotSubscribedException e) {
			// TBD: Die Exception ist wiedersprüchlich dokumentiert --> Roland
			LogTools.log(debug, LmsMsg.ErrorAnmeldungEmpfang, dataDescription);
			throw new RuntimeException(e);
		}
	}

	/**
	 * &Uuml;berpr&uuml;ft alle Dateien, ob sie gel&ouml;scht werden
	 * k&ouml;nnen. Eine Meldungsdatei wird gel&ouml;scht, wenn alle in ihr
	 * enthalten Meldungen abgelaufen sind.
	 */
	private void ueberpruefeDateien() {
		final List<File> loeschliste = new ArrayList<File>();

		for (final File datei : dateiMap.keySet()) {
			boolean loeschen = true;
			final List<RDSMeldung> liste = dateiMap.get(datei);
			for (final RDSMeldung meldung : liste) {
				if (meldungen.containsValue(meldung) && meldung.istZeitGueltig()) {
					loeschen = false;
				}
			}

			if (loeschen) {
				loeschliste.add(datei);
			}
		}

		// Dateien löschen
		for (final File datei : loeschliste) {
			LogTools.log(debug, LmsMsg.FineDateiGeloeschtAbgelaufen, datei);
			datei.delete();
			dateiMap.remove(datei);
		}
	}

	/**
	 * &Uuml;berp&uuml;ft das Sicherungsverzeichnis und l&ouml;scht alle
	 * Dateien, die l&auml;nger als die parametrierte Sicherungszeit nicht
	 * ge&auml;ndert wurden.
	 */
	private void ueberpruefeSicherungsverzeichnis() {
		final File sicherungsverzeichnis = new File(
				LMSProperties.getInstance().getProperty(LMSProperties.PROP_KEY_SICHERUNGSVERZEICHNIS));

		if (sicherungsverzeichnis.length() == 0) {
			// keine Sicherung
			return;
		}

		// Sicherungszeit ist in Tagen
		final long sicherungszeit = Long
				.parseLong(LMSProperties.getInstance().getProperty(LMSProperties.PROP_KEY_SICHERUNGSZEIT))
				* Constants.MILLIS_PER_DAY;

		try {
			if (sicherungsverzeichnis.exists()) {
				final File[] dateien = sicherungsverzeichnis.listFiles();

				if (dateien != null) {
					for (final File datei : dateien) {
						if ((System.currentTimeMillis() - datei.lastModified()) > sicherungszeit) {
							datei.delete();
						}
					}
				}
			}
		} catch (final Exception e) {
			LogTools.log(debug, LmsMsg.ErrorAppException, e.getMessage());
		}
	}

	@Override
	public void update(final ResultData[] results) {
		for (final ResultData datensatz : results) {
			if (datensatz.hasData()) {
				if (datensatz.getDataDescription().equals(paramEmpfang)) {
					debug.info("Parameterdatensatz erhalten für ",
							datensatz.getObject() + ", " + datensatz.getDataDescription().getAttributeGroup());
					lmsParameter.aktualisieren(datensatz.getData());
					synchronized (this) {
						this.notify();
					}
				}
			}
		}
	}

	/**
	 * Verarbeitet neu empfangene Dateien.
	 */
	private void verarbeiteNeueDateien() {

		ArrayList<File> dateien = null;

		synchronized (neueDateien) {
			if (!neueDateien.isEmpty()) {
				dateien = new ArrayList<File>(neueDateien);
				neueDateien.clear();
			}
		}

		if ((dateien != null) && (dateien.size() > 0)) {
			for (final File datei : dateien) {
				LogTools.log(debug, LmsMsg.InfoDateiVerarbeitet, datei);

				final List<RDSMeldung> dateimeldungen = new ListeOhneNullElemente<RDSMeldung>();

				final TICInfoDatei tid = new TICInfoDatei(datei);
				boolean fehler = false;

				try {
					for (final RDSMeldungsDaten meldung : tid.bildeMeldungen()) {
						if (istRelevant(meldung) && meldung.istZuVerarbeiten()) {
							try {
								if (meldungen.containsKey(meldung.bildePid())) {
									dateimeldungen.add(aktualisiereMeldung(meldung));
								} else {
									dateimeldungen.add(neueMeldung(meldung));
								}
							} catch (final Exception e) {
								final String msg = e.getMessage() + " (Meldung: " + meldung.getIdentifikation()
										+ ", Datei: " + datei + ")";
								LogTools.log(debug, LmsMsg.ErrorAppException, msg);
								LogTools.log(debug, LmsMsg.ErrorAppException, LoggerTools.getStackTrace(e));
								fehler = true;
							}
						} else {
							LogTools.log(debug, LmsMsg.FineMeldungUngueltig, meldung.bildePid());
						}
					}
				} catch (final TICInfoDateiException te) {
					LogTools.log(debug, LmsMsg.ErrorTICDatei, datei, te.getMessage());
					sichereDatei(datei);
				}

				if (fehler) {
					sichereDatei(datei);
				}

				if (!dateimeldungen.isEmpty()) {
					dateiMap.put(datei, dateimeldungen);
				} else {
					// keine relevanten Meldungen in der Datei
					LogTools.log(debug, LmsMsg.FineDateiGeloescht, datei);
					datei.delete();
				}
			}
		}
	}

	@Override
	public void versendet(final List<File> dateien) {
		/** im Dekodierer nichts zu tun */
	}

	@Override
	public void invalidObject(final DynamicObject dynamicObject) {
		debug.info("Meldung: " + dynamicObject.getPid() + " wurde invalidiert!");
		entfernenMeldung(dynamicObject);
	}

}
