/*
 * Segment 2 (KEx),  SWE 2.1 LMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lms.vew.daten;

import org.w3c.dom.Node;

import de.bsvrz.kex.lms.vew.RDSException;

/**
 * RDS-Locationcode.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */

public class RDSLocationCode extends RDSInteger {

	/**
	 * TIC-Info Specification:
	 * "4294967295 is a dummy code for locations with LocationFormat".
	 */
	private static final long TIC_LOCATION_DUMMY_WERT = 4294967295L;

	/**
	 * Erzeugt ein neues Objekt.
	 *
	 * @param xmlName
	 *            Name des XML-ELementes
	 * @param davName
	 *            Name des DAV-Attributes
	 */
	public RDSLocationCode(final String xmlName, final String davName) {
		super(xmlName, davName);
	}

	/**
	 * Das XML-Element 'LocationLocCode' kann auch den Wert:<br>
	 * <br>
	 * <code>4294967295 is a dummy code for locations with LocationFormat = TIC Location.</code>
	 * <br>
	 * <br>
	 * annehmen.
	 */
	@Override
	public void getXmlDaten(final Node node) throws RDSException {
		final long lwert = Long.parseLong(node.getTextContent());

		if (lwert == RDSLocationCode.TIC_LOCATION_DUMMY_WERT) {
			wert = 0;
		} else if ((lwert > 65535) || (lwert < 0)) {
			throw new RDSException("Der empfangene Wert '" + lwert + "' des XML-Elementes '" + getXmlElementName()
					+ "' für das Attribut '" + getDavAttributName() + "' ist ungültig");
		} else {
			wert = (int) lwert;
		}
	}
}
